/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.security.idm;

//core java
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.ArrayList;
import java.util.StringTokenizer;

//core java security
import java.security.Principal;
import java.security.acl.Group;

import javax.security.auth.Subject;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.login.LoginException;



//core jboss security
import org.jboss.security.auth.spi.UsernamePasswordLoginModule;
import org.jboss.security.SimpleGroup;


/**
 * A built-in JAAS authentication module that supports authentication via username and password</br></br>
 * 
 * It makes use of the LoginProvider, so developers shouldn't need to implement the authentication logic in multiple places
 * 
 * @author Sohil Shah - sohil.shah@jboss.com - Jun 12, 2006
 *
 */
public class UsernameAndPasswordLoginModule extends UsernamePasswordLoginModule
{
    Collection authenticatedRoles = new ArrayList();
    private LoginProvider provider = null;
    
    /**
     * Initializes the JAAS Module before it can start accepting authentication requests
     * 
     * @param subject
     * @param callbackHandler
     * @param sharedState
     * @param options
     */
    public void initialize(Subject subject, CallbackHandler callbackHandler,Map sharedState, Map options)
    {
        super.initialize(subject,callbackHandler,sharedState,options);
        
        //setup the authenticatedRoles... this is a comma-separated list
        String authenticatedRoles = (String)options.get("authenticatedRoles");
        if(authenticatedRoles!=null && authenticatedRoles.trim().length()>0)
        {
            StringTokenizer st = new StringTokenizer(authenticatedRoles,","); 
            while(st.hasMoreTokens())
            {
                this.authenticatedRoles.add(st.nextToken());
            }
        }
        
        //use the specified provider
        String providerId = (String)options.get("provider");
        if(providerId == null || providerId.trim().length()==0)
        {
            provider = IdentityManager.findLoginProvider();
        }
        else
        {
            provider = IdentityManager.findLoginProvider(providerId);
        }
    }
    
    /**
     * Provides the default implementation to perform the authentication using username and password. 
     * For custom authentication usecases, developers can extend this
     * method and provide the proper implementation
     * 
     * @return password registered in the system
     */
    protected String getUsersPassword() throws LoginException
    {
        try
        {
            String encryptedPassword = null;
                                    
            if(
                    this.provider.exists(this.getUsername())
             )
            {
                Identity identity = this.provider.read(this.getUsername());
                
                //also make sure the user is active
                if(!identity.isActive())
                {
                    //signify to the system that this account is inactive                                       
                    throw new LoginException("The specified account has not been activated-"+this.getUsername());
                }
                                                                
                encryptedPassword = new String(identity.getPassword());
            }
                        
            return encryptedPassword;
        }
        catch(Exception e)
        {
            log.error(this,e);
            throw new LoginException(e.toString());            
        }
    }
    
        
    /**
     * Provides the default implementation to get the Groups/Roles that the user belongs to. 
     * For custom authentication usecases, developers can extend this
     * method and provide the proper implementation
     * 
     * @return an array of Groups that this user belongs to 
     * @throws LoginException
     */
    protected Group[] getRoleSets() throws LoginException
    {
        try
        {
            Group[] groups = {new SimpleGroup("Roles")};
                 
            if(
                    this.provider.exists(this.getUsername())
             )
            {
                Identity identity = this.provider.read(this.getUsername()); 
                
                if(this.authenticatedRoles!=null)
                {
                    for(Iterator itr=this.authenticatedRoles.iterator();itr.hasNext();)
                    {
                        Principal role = this.createIdentity((String)itr.next());
                        groups[0].addMember(role);
                    }
                }
                
                //grab the roles associated with this user
                Collection roles = identity.getRoles(this.provider);
                if(roles!=null)
                {
                    for(Iterator itr=roles.iterator();itr.hasNext();)
                    {
                        Principal role = this.createIdentity((String)itr.next());                  
                        groups[0].addMember(role);
                    }
                }
                
            }        
            
            return groups;
        }
        catch(Exception e)
        {
            log.error(this,e);
            throw new LoginException(e.toString());            
        }
    }        
}
