/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.security.saml;

import java.util.Properties;
import java.util.Map;

/**
 * SingleSignOn interface is used to marshal/unmarshal SSO tokens.<br/><br/>
 * 
 * The interface makes no assumptions about the format of the SSO token. It is upto the implementations of this
 * interface to produce the tokens in a format they desire. <br/>
 * 
 * The JBoss SSO system ships with a SingleSignOn implementation that produces the tokens in a SAML format.<br/>
 * 
 * @author Sohil Shah - sohil.shah@jboss.com
 *
 */
public interface SingleSignOn 
{
    /**
     * name of the SSO domain level cookie
     */
    public static final String SSO_TOKEN = "token"; //name of the SSO domain level cookie
    
    /**
     * name of the SSO logout in progress cookie
     */
    public static final String LOGOUT_TOKEN = "logoutToken"; //name of the logoutInProgress cookie
    
    /**
     * name of the logout destination cookie
     */
    public static final String LOGOUT_DEST = "logoutDest"; //name of the logout destination cookie
    
    /**
     * name of the username request attribute
     */
    public static final String SSO_USERNAME = "jboss_sso_username"; //username request attribute
    
    /**
     * name of the password request attribute
     */
    public static final String SSO_PASSWORD = "jboss_sso_password"; //password request attribute
    
    /**
     * name of the established SSO session's attribute
     */
    public static final String SSO_SESSION = "jboss_sso_session"; //SSOSession session attribute
    
    /**
     * name if the SSO User authenticated on the system. This is used by partner applications
     * that use a non-JAAS authentication method
     */
    public static final String SSO_USER = "jboss_sso_user"; //SSOUser session attribute to be used by non-JAAS authentication usecases
    
    
    
    /**
     * This method is used to set configuration properties on the Single Sign On implementation
     * 
     * @param properties
     */
    public void setProperties(Properties properties);
    
    
    /**
     * This method generates a SAML authentication request based on the supplied username and password
     * 
     * @param username
     * @param password
     * @return
     * @throws SSOException
     */
    public String generateAuthRequest(String username,String password) 
    throws SSOException;
    
    /**
     * This method parses a SAML authentication request into a SSOUser domain object
     * 
     * @param request
     * @return
     * @throws SSOException
     */
    public SSOUser parseAuthRequest(String request) throws SSOException;
    
    /**
     * This method generates a SAML authentication response based on the supplied username, password, and the 
     * status of the authentication process
     * 
     * @param assertingParty
     * @param username
     * @param password
     * @param success
     * @return
     * @throws SSOException
     */
    public String generateAuthResponse(String assertingParty,String username,boolean success) 
    throws SSOException;
    
    /**
     * This method parses a SAML authentication response and produces an AuthResponse domain object
     * 
     * @param response
     * @return
     * @throws SSOException
     */
    public AuthResponse parseAuthResponse(String response)
    throws SSOException;
    
    /**
     * This method returns a SAML (xml) representation of a map of partners that are part of
     * this Single Sign On federation
     * 
     * @param partners - a map of partners belonging to the SSO Federation
     * @return
     * @throws SSOException
     */
    public String marshalPartners(Map partners)
    throws SSOException;
    
    /**
     * This method unmarshals a SAML (xml) representation of partners that are part of this Single Sign On
     * Federation 
     * It returns a map of these partners with
     * key - domain of the partner
     * value - the address of the federation server of this partner
     * 
     * @return
     * @throws SSOException
     */
    public Map unmarshalPartners(String partners)
    throws SSOException;
}
